@@ -10,7 +10,7 @@
 ##' @param ... additional parameter
 ##' @return ggplot2 layer
 ##' @export
-##' @author guangchuang yu
+##' @author Guangchuang Yu
 geom_pokemon <- function(mapping=NULL, data=NULL, inherit.aes=TRUE,
                        na.rm=FALSE, by="width", ...) {
     geom_image(mapping, data, inherit.aes=inherit.aes, na.rm=na.rm, ..., .fun = pokemon)
@@ -28,9 +28,11 @@ pokemon <- function(id) {
 ##' @title list.pokemon
 ##' @return pokemon vector
 ##' @export
-##' @author guangchuang yu
+##' @author Guangchuang Yu
 list.pokemon <- function() {
-    list.png("https://github.com/Templarian/slack-emoji-pokemon/tree/master/emojis")
+    #list.png("https://github.com/Templarian/slack-emoji-pokemon/tree/master/emojis")
+    files <- get_github_files("Templarian", "slack-emoji-pokemon", "emojis", "master")
+    sub(".png", "", files)
 }
 
 list.png <- function(url) {
